@@ -1,6 +1,6 @@
 /* font-manager-preview-page.c
  *
- * Copyright (C) 2009-2024 Jerry Casiano
+ * Copyright (C) 2009-2025 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,6 +116,7 @@ struct _FontManagerPreviewPage
     GtkWidget   *textview;
     GtkWidget   *menu_button;
 
+    gint                line_spacing;
     gdouble             waterfall_size_ratio;
     gdouble             min_waterfall_size;
     gdouble             max_waterfall_size;
@@ -141,6 +142,7 @@ enum
     PROP_WATERFALL_MAX,
     PROP_WATERFALL_RATIO,
     PROP_SHOW_LINE_SIZE,
+    PROP_LINE_SPACING,
     N_PROPERTIES
 };
 
@@ -199,6 +201,9 @@ font_manager_preview_page_get_property (GObject    *gobject,
         case PROP_WATERFALL_RATIO:
             g_value_set_double(value, self->waterfall_size_ratio);
             break;
+        case PROP_LINE_SPACING:
+            g_value_set_int(value, self->line_spacing);
+            break;
         case PROP_SHOW_LINE_SIZE:
             g_value_set_boolean(value, self->show_line_size);
             break;
@@ -241,6 +246,11 @@ font_manager_preview_page_set_property (GObject      *gobject,
         case PROP_WATERFALL_RATIO:
             font_manager_preview_page_set_waterfall_size(self, -1.0, -1.0, g_value_get_double(value));
             break;
+        case PROP_LINE_SPACING:
+            self->line_spacing = g_value_get_int(value);
+            gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(self->textview), self->line_spacing);
+            gtk_text_view_set_pixels_below_lines(GTK_TEXT_VIEW(self->textview), self->line_spacing);
+            break;
         case PROP_SHOW_LINE_SIZE:
             self->show_line_size = g_value_get_boolean(value);
             if (self->mode == FONT_MANAGER_PREVIEW_PAGE_MODE_WATERFALL)
@@ -386,6 +396,19 @@ font_manager_preview_page_class_init (FontManagerPreviewPageClass *klass)
                                                                 G_PARAM_STATIC_STRINGS |
                                                                 G_PARAM_READWRITE);
 
+    /**
+     * FontManagerPreviewPage:line-spacing:
+     *
+     * Pixels between lines in Waterfall preview.
+     */
+    obj_properties[PROP_LINE_SPACING] = g_param_spec_int("line-spacing",
+                                                         NULL,
+                                                         "Waterfall preview line spacing",
+                                                         0,
+                                                         G_MAXINT,
+                                                         0,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
     g_object_class_install_properties(object_class, N_PROPERTIES, obj_properties);
     return;
 }
@@ -452,7 +475,7 @@ generate_waterfall_preview (FontManagerPreviewPage *self)
     gtk_text_buffer_set_text(buffer, "", -1);
     g_idle_remove_by_data(self);
     current_line = self->min_waterfall_size;
-    g_idle_add((GSourceFunc) generate_waterfall_line, self);
+    g_idle_add_full(G_PRIORITY_HIGH_IDLE, (GSourceFunc) generate_waterfall_line, self, NULL);
     return;
 }
 
@@ -639,13 +662,13 @@ font_manager_preview_page_init (FontManagerPreviewPage *self)
     g_autoptr(GtkTextTagTable) tag_table = font_manager_text_tag_table_new();
     self->pangram = font_manager_get_localized_pangram();
     self->default_pangram = font_manager_get_localized_pangram();
-    self->preview = g_strdup_printf(FONT_MANAGER_DEFAULT_PREVIEW_TEXT, self->pangram);
+    self->preview = font_manager_get_localized_preview_text();
     self->default_preview = g_strdup(self->preview);
     self->justification = GTK_JUSTIFY_CENTER;
     g_autoptr(GtkTextBuffer) buffer = gtk_text_buffer_new(tag_table);
     GtkWidget *scroll = gtk_scrolled_window_new();
     self->textview = gtk_text_view_new_with_buffer(buffer);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(self->textview), FALSE);
+    /* gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(self->textview), FALSE); */
     GtkWidget *controls = font_manager_preview_controls_new();
     self->controls = gtk_revealer_new();
     GtkWidget *fontscale = font_manager_font_scale_new();
@@ -957,6 +980,7 @@ font_manager_preview_page_set_waterfall_size (FontManagerPreviewPage *self,
  *  - min-waterfall-size
  *  - max-waterfall-size
  *  - waterfall-size-ratio
+ *  - waterfall-line-spacing
  */
 void
 font_manager_preview_page_restore_state (FontManagerPreviewPage *self,
@@ -968,10 +992,19 @@ font_manager_preview_page_restore_state (FontManagerPreviewPage *self,
     g_settings_bind(settings, "preview-font-size", self, "preview-size", flags);
     g_settings_bind(settings, "preview-mode", self, "preview-mode", flags);
     g_settings_bind(settings, "preview-text", self, "preview-text", flags);
+    // Our settings use the English version as a default...
+    PangoLanguage *lang = pango_language_from_string("xx");
+    const gchar *pangram = pango_language_get_sample_string(lang);
+    g_autofree gchar *xx = g_strdup_printf(FONT_MANAGER_DEFAULT_PREVIEW_TEXT, pangram);
+    // If the stored version matches the English default use the localized version
+    if (g_strcmp0(self->preview, xx) == 0)
+        font_manager_preview_page_set_preview_text(self, self->default_preview);
     g_settings_bind(settings, "waterfall-show-line-size", self, "show-line-size", flags);
+    g_settings_bind(settings, "waterfall-line-spacing", self, "line-spacing", flags);
     g_settings_bind(settings, "min-waterfall-size", self, "min-waterfall-size", flags);
     g_settings_bind(settings, "max-waterfall-size", self, "max-waterfall-size", flags);
     g_settings_bind(settings, "waterfall-size-ratio", self, "waterfall-size-ratio", flags);
+    font_manager_preview_page_set_waterfall_size(self, -1.0, -1.0, -1.0);
     return;
 }
 
